#include <Wire.h>
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <ArduinoJson.h>
#include <TimeLib.h> // Include the Time library
#include <LiquidCrystal_I2C.h>
#include <EEPROM.h>
#include <WiFiManager.h>

// API Link Variable
String apiLink = "https://api.aladhan.com/v1/timingsByAddress/DD-MM-YYYY?address=Amman,jordan&method=1&tune=0,0,0,0,2,7,0,0,0&adjustment=1";

// Define digital pins for each prayer and reset button
const int athanPin = 3;
const int resetPin = 1; // GPIO pin for reset button

// Timing Variables
unsigned long startTime = 0; // Variable to store the start time for each prayer
unsigned long duration = 196000; // Duration for each prayer (3 minutes and 16 seconds in milliseconds)

// I2C LCD Configuration
const byte SDA_PIN = 0;  // GPIO0
const byte SCL_PIN = 2;  // GPIO2
const int LCD_ADDR = 0x27; // I2C address of the LCD
const int LCD_COLS = 20;   // Number of columns in the LCD
const int LCD_ROWS = 4;    // Number of rows in the LCD
LiquidCrystal_I2C lcd(LCD_ADDR, LCD_COLS, LCD_ROWS);

unsigned long previousMillis = 0; // Variable to store the previous time
const long interval = 10000; // Interval between switching screens (10 seconds)
bool displayFirstScreen = true; // Flag to track which screen to display

WiFiManager wm; // WiFiManager instance

void setup() {
  Serial.begin(115200);
  Wire.begin(SDA_PIN, SCL_PIN);
  pinMode(athanPin, OUTPUT);

  // Initialize EEPROM
  EEPROM.begin(512);

  // Load API Link from EEPROM
  loadApiLinkFromEEPROM();
  Serial.println("Loaded API link: " + apiLink);

  // Initialize Reset Button
  pinMode(resetPin, INPUT_PULLUP);

  // Initialize LCD
  lcd.begin();
  lcd.backlight(); // Turn on the backlight

  // Check if Wi-Fi is already configured
  if (WiFi.SSID().isEmpty()) {
    // No Wi-Fi credentials saved, display instructions
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(" Open WiFi settings");
    lcd.setCursor(0, 1);
    lcd.print("   and connect to   ");
    lcd.setCursor(0, 2);
    lcd.print("'Prayer Times Clock'");
    lcd.setCursor(0, 3);
    lcd.print(" to complete setup. ");
  } else {
    // Wi-Fi is configured, display connecting message
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(" Connecting to WiFi");
  }

  // WiFiManager Custom Parameters
  WiFiManagerParameter customApiLink("apiLink", "API Link", apiLink.c_str(), 128);
  WiFiManagerParameter timeZoneParam("timeZone", "UTC/GMT Offset  (e.g., +3)", "3", 4);

  // Add Parameters
  wm.addParameter(&customApiLink);
  wm.addParameter(&timeZoneParam);

  // Start WiFiManager
  if (!wm.autoConnect("Prayer Times Clock")) {
    Serial.println("Failed to connect or timeout.");
    ESP.restart();
  }

  // Save API Link and Time Zone Offset
  apiLink = customApiLink.getValue();
  saveApiLinkToEEPROM(apiLink);
  String timeZoneOffset = timeZoneParam.getValue();
  saveTimeZoneToEEPROM(timeZoneOffset);

  // Load saved time zone and configure NTP
  String savedTimeZone = loadTimeZoneFromEEPROM();
  int timeZoneInSeconds = savedTimeZone.toInt() * 3600;
  configTime(timeZoneInSeconds, 0, "pool.ntp.org");

  // Wait for time to synchronize
  while (!time(nullptr)) {
    Serial.println("Waiting for NTP sync...");
    delay(1000);
  }
  Serial.println("NTP time synchronized!");

  // Display connection success
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(" Connected to WiFi");
  delay(2000);
}







void loop() {
  // Handle Wi-Fi reconnection
  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("Reconnecting to WiFi...");

    
    // Attempt reconnection using WiFiManager
    if (!wm.autoConnect("Prayer Times Clock")) {
      Serial.println("WiFi connection failed. Restarting...");
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("WiFi connect failed");
      delay(2000); // Show error for 2 seconds
      ESP.restart();
    } else {
      Serial.println("WiFi reconnected!");
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print(" Connected to WiFi");
      delay(2000); // Display confirmation for 2 seconds
    }
  }

  // Handle Reset Button Logic
  if (digitalRead(resetPin) == LOW) {
    delay(50); // Initial debounce
    if (digitalRead(resetPin) == LOW) {
      unsigned long pressedTime = millis();
      while (digitalRead(resetPin) == LOW) {
        if (millis() - pressedTime > 1000) { // Long press detected
          Serial.println("Resetting WiFi settings...");
          lcd.clear();
          lcd.setCursor(0, 0);
          lcd.print(" Resetting WiFi...");
          delay(3000);
          wm.resetSettings(); // Clear saved WiFi credentials
          lcd.clear();
          lcd.setCursor(0, 0);
          lcd.print("WiFi settings reset");
          delay(3000);
          ESP.restart();
        }
      }
    }
  }

  // Handle screen switching
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis; // Reset timer
    displayFirstScreen = !displayFirstScreen; // Toggle screens

    // Fetch and display prayer times
    fetchPrayerTimes();
  }

  delay(100); // Small delay to avoid CPU overload
}



void fetchPrayerTimes() {
  WiFiClient client;
  HTTPClient http;

  if (http.begin(client, apiLink)) {
    int httpCode = http.GET();
    if (httpCode > 0) {
      if (httpCode == HTTP_CODE_OK) {
        String payload = http.getString();
        deserializeJson(payload);
      }
    }
    http.end();
  }
}

void deserializeJson(String payload) {
  DynamicJsonDocument doc(1024);
  deserializeJson(doc, payload);

  JsonObject times = doc["data"]["timings"];
  JsonObject data = doc["data"];
  JsonObject date = data["date"];
  JsonObject hijri = date["hijri"];
  JsonObject gregorian = date["gregorian"];
  String gregorianDayName = gregorian["weekday"]["en"];
  String gregorianDate = gregorian["date"];
  int hijriMonthNumber = hijri["month"]["number"];
  String hijriDayNumber = hijri["day"];
  String hijriYear = hijri["year"];
  String fajr = String(times["Fajr"].as<String>());
  String dhuhr = String(times["Dhuhr"].as<String>());
  String asr = String(times["Asr"].as<String>());
  String maghrib = String(times["Maghrib"].as<String>());
  String isha = String(times["Isha"].as<String>());

  String currentTime = getCurrentTime(); // Get the current time

  // Call the display functions with the retrieved prayer times
  if (displayFirstScreen) {
    displayFirstScreenContent(dhuhr, asr, maghrib, isha, fajr);
  } else {
    String hijriMonthName = getHijriMonthName(hijriMonthNumber);
    displaySecondScreenContent(gregorianDayName, hijriDayNumber, hijriMonthName, hijriYear, currentTime, gregorianDate);
  }

  // Check if it's time for the athan and trigger it
if (currentTime == fajr || currentTime == dhuhr || currentTime == asr || currentTime == maghrib || currentTime == isha) {
  displayAthanMessage(); // Display Athan message
  digitalWrite(athanPin, HIGH);
  delay(duration);
  digitalWrite(athanPin, LOW);
  lcd.clear(); // Clear the screen after Athan
}
}

String getCurrentTime() {
  struct tm timeinfo;
  if (!getLocalTime(&timeinfo)) {
    return "00:00"; // Return default time if unable to fetch current time
  }

  char currentTime[6]; // Enough to hold "HH:MM\0"
  sprintf(currentTime, "%02d:%02d", timeinfo.tm_hour, timeinfo.tm_min);
  return String(currentTime);
}

void displayFirstScreenContent(String dhuhr, String asr, String maghrib, String isha, String fajr) {
  lcd.clear();
  int maxChars = 20; // Maximum characters per line on the LCD
  int startingColumn; // Variable to store the starting column for centering
  
  lcd.setCursor(0, 0);
  lcd.print("Duhr " + dhuhr + " " + "Asr " + asr);
 
  startingColumn = (int)std::max(0, (int)(maxChars - 8 - maghrib.length()) / 2); // Calculate starting column for centering
  lcd.setCursor(startingColumn, 1);
  lcd.print("Maghrib " + maghrib);
  
  startingColumn = (int)std::max(0, (int)(maxChars - 5 - isha.length()) / 2); // Calculate starting column for centering
  lcd.setCursor(startingColumn, 2);
  lcd.print("Isha " + isha);
  
   startingColumn = (int)std::max(0, (int)(maxChars - 5 - fajr.length()) / 2); // Calculate starting column for centering
  lcd.setCursor(startingColumn, 3);
  lcd.print("Fajr " + fajr);
}

void displaySecondScreenContent(String gregorianDayName, String hijriDayNumber, String hijriMonthName, String hijriYear, String currentTime, String gregorianDate) {
  lcd.clear();
  int maxChars = 20; // Maximum characters per line on the LCD
  int startingColumn = (int)std::max(0, (int)(maxChars - gregorianDayName.length()- gregorianDate.length()) / 2); // Calculate starting column for centering
  
  lcd.setCursor(startingColumn, 0);
  lcd.print(gregorianDayName + " " +  gregorianDate);
  
  startingColumn = (int)std::max(0, (int)(maxChars - (hijriDayNumber.length() + hijriMonthName.length() + 1)) / 2); // Calculate starting column for centering
  lcd.setCursor(startingColumn, 1);
  lcd.print(hijriDayNumber + " " + hijriMonthName);
  Serial.println(hijriDayNumber);
  startingColumn = (int)std::max(0, (int)(maxChars - hijriYear.length()) / 2); // Calculate starting column for centering
  lcd.setCursor(startingColumn, 2);
  lcd.print(hijriYear);
  
  startingColumn = (int)std::max(0, (int)(maxChars - 9 - currentTime.length()) / 2); // Calculate starting column for centering
  lcd.setCursor(startingColumn, 3);
  lcd.print("Time now " + currentTime);
}

String getHijriMonthName(int monthNumber) {
  switch (monthNumber) {
    case 1: return "Muharram";
    case 2: return "Safar";
    case 3: return "Rabi' al-Awwal";
    case 4: return "Rabi' al-Thani";
    case 5: return "Jumada al-Awwal";
    case 6: return "Jumada al-Thani";
    case 7: return "Rajab";
    case 8: return "Sha'ban";
    case 9: return "Ramadan";
    case 10: return "Shawwal";
    case 11: return "Dhul-Qi'dah";
    case 12: return "Dhul-Hijjah";
    default: return "Unknown";
  }
}

void saveApiLinkToEEPROM(String apiLink) {
  for (int i = 0; i < apiLink.length(); i++) {
    EEPROM.write(i, apiLink[i]);
  }
  EEPROM.write(apiLink.length(), '\0');
  EEPROM.commit();
}

void loadApiLinkFromEEPROM() {
  String loadedApiLink = "";
  for (int i = 0; i < 128; i++) {
    char c = EEPROM.read(i);
    if (c == '\0') break;
    loadedApiLink += c;
  }
  apiLink = loadedApiLink;
}

void saveTimeZoneToEEPROM(String timeZoneOffset) {
  for (int i = 0; i < timeZoneOffset.length(); i++) {
    EEPROM.write(128 + i, timeZoneOffset[i]); // Save at offset 128 to avoid overwriting API link
  }
  EEPROM.write(128 + timeZoneOffset.length(), '\0'); // Null terminator
  EEPROM.commit();
}

String loadTimeZoneFromEEPROM() {
  String loadedTimeZone = "";
  for (int i = 128; i < 132; i++) { // Assuming max 4 characters (+XX or -XX)
    char c = EEPROM.read(i);
    if (c == '\0') break;
    loadedTimeZone += c;
  }
  return loadedTimeZone;
}

void displayAthanMessage() {
  lcd.clear();
  lcd.setCursor(0, 0); // Adjust position as necessary
  lcd.print("Athan in progress...");
  lcd.setCursor(0, 1);
  lcd.print("Please take time to");
  lcd.setCursor(0, 2);
  lcd.print("perform your prayer.");
  lcd.setCursor(0, 3);
  lcd.print(" Peace be upon you.");
} 
